/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_DELETELIST
#define VORLIB_DELETELIST

namespace vorlib {

template <class t> class deletelist {
private:
  int allocated;//how much do we have allocated
  int count;//how many used
  t **list;//and the list itself
public:
  deletelist();//implicit constructor
  ~deletelist();//destructor
  deletelist &operator<<(t *value);//include one more to the list
};
/*
 this is list of dynamically created objects (of one type)
 they will be deleted automatically when the deletelist is deleted
 for outodestroy managment of objects
*/

template <class t> deletelist<t>::deletelist() {
  list = NULL;
  allocated = 0;
  count = 0;//set them all empty
}

template <class t> deletelist<t>::~deletelist() {
  t **end = list + count;//one after the last
  for (t **pos = list; pos != end; pos ++)
    delete *pos;//delete them all
  free(list);//and lear the array
}

template <class t> deletelist<t> &deletelist<t>::operator<<(t *value) {
  if (allocated == count) // the array is full, make it bigger
    list = (t **) realloc(list, (allocated += 10) * sizeof *list);//make it 10 bigger
  list[count ++] = value;//copy the pointer
  return *this;
}

}

#endif
