/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "filelist.h"
#include <vector>
#include <stdio.h>

using std::string;
using std::vector;

namespace vorlib {

filelist_e::filelist_e(const string &message, int id) : exception(id, message) {
}

bool filelist::section::add(const string &name, const string &pth, int convert) {
  string path = pth;
  switch (convert) {
    case 0://do nothing, no convert
      break;
    case 1:/* / -> \ */
      for (string::iterator c = path.begin(); c != path.end(); c++) if (*c == '/') *c = '\\';
      break;
    case 2:/* \ -> / */
      for (string::iterator c = path.begin(); c != path.end(); c++) if (*c == '\\') *c = '/';
      break;
  }
  return files.add(name, path);//just put it there
}

const string &filelist::section::operator[](const string &name) const {
  return files[name];
}

stringlist filelist::section::all() const {
  vector<string> l = files.get_data();
  stringlist res;
  for (vector<string>::iterator s = l.begin(); s != l.end(); s++) res += *s;
  return res;
}

void filelist::clear() {
  sections.clear();//no sections -> no items
}

void filelist::load(const string &file, int convert) {
  FILE *f = fopen(file.c_str(), "rt");
  if (!f) throw new filelist_e("file not found:" + file, 1);
  char *line = new char[10240];//it should be enough
  char *l;
  section *actsec = NULL;//no section yet
  while ((l = fgets(line, 10240, f)) != NULL) {
    if (*l == '#') continue;//it's comment
    if (l[strlen(l) - 1] == '\n') l[strlen(l) - 1] = 0;
    if (*l == '[') {
      for (char *c = l + strlen(l) - 1; l != c; c++) if (*c == ']') {
        *c = 0;//cut it now
        l ++;//start after [
        string sec = l;//get the string
        section actual;
        if (!sections.add(sec, actual)) throw new filelist_e("section duplicated:" + sec, 3);//add the section
        actsec = &sections[sec];//and get pointer to actual section
        goto FOUND_SECTION;//new line, please
      }
      throw new filelist_e("synatx error, ']' not found", 4);//not found the ']'
      FOUND_SECTION:
      continue;
    }
    for (char *c = l; *c; c++) if (*c == ' ') {
      *c = 0;
      char *path = c + 1;
      if (!*path) throw new filelist_e("path missing", 6);
      string name = l;//the name is at the beginign of the line
      string pth = path;
      if (!actsec) throw new filelist_e("path outside any section", 7);
      if (!actsec -> add(name, pth, convert)) throw new filelist_e("path name duplicated", 8);
      goto FOUND_PATH;
    }
    throw new filelist_e("syntax error, space missing after file name", 5);
    FOUND_PATH:
    continue;
  }                                                                
  delete line;//not needed
  if (fclose(f)) throw new filelist_e("couldn't close file:" + file, 2);
}

stringlist filelist::getsection(const string &section) const {
  try {
    stringlist s = sections[section].all();//get the paths
    for (int i = 0; i < s.count(); i++) {//and connect them all
      s[i] = connectpath(s[i]);
    }
    return s;//and return it all
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new filelist_e("section does not exist", 10);
    } else throw e;//just send it there
  }
}

string filelist::getpath(const string &section, const string &name) const {
  try {
    return connectpath(sections[section][name]);//get the path, connect it with base path and return it
  }
  catch (tree_e *e) {
    if (e -> id == 1) {//it means no such.. and so on
      delete e;
      throw new filelist_e("section or path does not exist");
    } else throw e;//some other error, I don't know and care, just send it away
  }
}

string filelist::connectpath(const string &pth) const {
  if (pth == "") throw new filelist_e("empty path", 11);
  if (isabsolute(pth)) return pth;//check, weather to connect it at all
  char *path = strdup(pth.c_str());//copy the path
  char *p = path;//save the position of memory block for free
  char *base = (basepath != "") ? strdup(basepath.c_str()) : NULL;//copy the base
  while ((path[0] == '.') && (path[1] == '.') && (path[2] == '/')) {//remove the '../' and step back in base
    if (!base) throw new filelist_e("empty base path, no slashes in it", 10);
    path += 3;//move it after the ../
    for (char *c = base + strlen(base) - 1; c != base; c--) if (*c == '/') {
      *(c + 1) = 0;//cut it after the slash
      goto SLASHFOUND;
    }
    throw new filelist_e("not enough forward slashes in base path", 10);
    SLASHFOUND:;
  }
  while ((path[0] == '.') && (path[1] == '.') && (path[2] == '\\')) {//for backslashes - the super win thing
    if (!base) throw new filelist_e("empty base path, no slashes in it", 10);
    path += 3;//move it after the ../
    for (char *c = base + strlen(base) - 1; c != base; c--) if (*c == '\\') {
      *(c + 1) = 0;//cut it after the slash
      goto BACKSLASHFOUND;
    }
    throw new filelist_e("not enough back slashes in base path", 10);
    BACKSLASHFOUND:;
  }
  string res = "";//make result
  if (base) res += base;
  res += path;//and put it together
  free(base);
  free(p);//tidy up
  return res;//amd return the part 
}

#ifdef VORLIB_UNIX//the linux part

bool filelist::isabsolute(const string &path) const {
  if (path.length() < 2) return false;//it is too short, it can't be absolute
  return ((path[0] == '/') && (path[1] != '/'));//it's absolute only if the first char is slash and the second isn't
}

#endif

#ifdef VORLIB_WINDOWS//the windows part

bool filelist::isabsolute(const string &path) const {
  if (path.length() < 3) return false;/*it is short enough for "X:\"*/
  return ((path[1] == ':') && (path[2] == '\\'));
}

#endif

}
