/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_SETUP_MANAGER
#define VORLIB_SETUP_MANAGER

#include <string>
#include "searchtree.h"
#include "stringlist.h"

namespace vorlib {

class setupmanager_e : public exception {
public:
  setupmanager_e(int id = 0, const std::string &message = "");
};

/*
 error IDs of setupmanager
 1: setup file not found
 2: file not closed
 3: duplicit section name
 4: ']' missing in section name
 5: '=' missing in value
 6: data out of section
 7: empty file name
 8: can not write to file
 9: value not of that type
 10: section or name not found
*/

class setupmanager {
private:
  typedef struct {
    std::string stringvalue;
    bool isint;
    int intvalue;
    bool isfloat;
    float floatvalue;
    bool isbool;
    bool boolvalue;
  } value_t;//this is one value, witch types it can be?

  typedef balanced_search_tree<std::string, value_t> section;//that means one section

  balanced_search_tree<std::string, section> sections;//all data there

  bool automanag;//do we use automanagment
  std::string file;//witch file with the setup it is?

  void loadfile(const std::string &name);//file > memory
  value_t processvalue(const std::string &value) const;//get what all typesit can be?
  value_t *tryget(const std::string &section, const std::string &name);//try to get the value or add it and return

public:
  setupmanager();//create empty
  setupmanager(const std::string &file, bool automanag = true);//create and set file
  ~setupmanager();//destroy (and save, if automanag)

  void setfile(const std::string &file, bool automanag = true);//set to the file and load if automanag
  void load();//load/reload from file
  void connectfiles(const std::string &file);//load one more file, set automanag to true and set the file to save to the new one
  void save() const;//save to file now

  const std::string &getstring(const std::string &section, const std::string &value) const;
  int getint(const std::string &section, const std::string &value) const;
  float getfloat(const std::string &section, const std::string &value) const;
  bool getbool(const std::string &section, const std::string &value) const;//get the specific value

  stringlist getSectionList() const;//returns list of sections in the setup manager
  stringlist getEntryList(const std::string &section) const;//returns list of all entries inside some section

  void setstring(const std::string &section, const std::string &value, const std::string &stringvalue);
  void setint(const std::string &section, const std::string &value, int intvalue);
  void setfloat(const std::string &section, const std::string &value, float floatvalue);
  void setbool(const std::string &section, const std::string &value, bool boolvalue);//set the specific value

  void remove(const std::string &section, const std::string &value);//this one won't be here
  void clear();//no setup's at all
};

/*
  it's setup file looks like this:

  [part 1]
  some_setup=value
  five=5
  string=some long string until the end of line
  float=87.023
  bool=true/false/yes/no/Y/N
  #comment until the end of line
  #note that all comments will be lost when save is called, or automanag used

  [part 2]
  next setup=blabla

  values can be of four types:
  string - all of them can be that
  bool - it is boolean value, it can have some of these:
    true/false
    yes/no
    Y/N
  float - for example 0.98, -35.009, 56.8e90 and many others
  int - 1234, -35 ...
*/

}

#endif
