/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef VORLIB_WINDOWS

//#include <windows.h>
#include <winsock2.h>

#define CLOSE(socket) closesocket(socket)
#define GET_ERROR strerror(WSAGetLastError())
#define H_ERRNO WSAGetLastError()
#define ERRNO WSAGetLastError()
//some defines to meke it a little more portable
#define EWOULDBLOCK WSAEWOULDBLOCK
#define EINPROGRESS WSAEINPROGRESS
#define socklen_t int FAR

#ifndef MSG_DONTWAIT
#define MSG_DONTWAIT 0
#endif

#ifndef MSG_WAITALL
#define MSG_WAITALL 0
#endif

#define ECONNRESET WSAECONNRESET
#define ESHUTDOWN WSAESHUTDOWN
#define ECONNABORTED WSAECONNABORTED

#endif

#ifdef VORLIB_UNIX

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/select.h>
#include <errno.h>
extern int h_errno;

#define CLOSE(socket) close(socket)
#define GET_ERROR strerror(errno)
#define H_ERRNO h_errno
#define ERRNO errno
#ifndef EWOULDBLOCK
#define EWOULDBLOCK EAGAIN
#endif

#ifndef ECONNABORTED
#define ECONNABORTED ECONNRESET
#endif

#ifndef ESHUTDOWN
#define ESHUTDOWN ECONNRESET
#endif

//a try to make it portable

#endif

#include "socket.h"//this is the vorlib socket header
#include <stdio.h>
#include <string.h>
#include <vector>
#include <stdarg.h>

namespace vorlib {

#ifdef VORLIB_WINDOWS

void set_non_block(int socket) {
  unsigned long value = 1;//the manual says it must be non-zero to be non blocking
  ioctlsocket(socket, FIONBIO, &value);//set it nonblocking
}

#define SETERROR(error) WSASetLastError(error)

#endif

#ifdef VORLIB_UNIX

void set_non_block(int socket) {
  fcntl(socket, F_SETFL, O_NONBLOCK);
}

#define SETERROR(error) errno = error

#endif

class server_internal_e : public exception {
public:
  server_internal_e(int code);
};//for internal server throwing

server_internal_e::server_internal_e(int code) : exception(code, "") {};

socket_e::socket_e(int id, const std::string &message, int _handle) : exception(id, message) {//this is the constructor of socket-exception
  handle = _handle;//set the handle as well
}

base_socket::base_socket() {//default constructor
  connected = false;//we are not connected
  hassock = false;//and we don't have socket yet
}

base_socket::~base_socket() {//socket destructor
  if (hassock) CLOSE(sockethandle);//close the socket if it is opened
}

bool base_socket::isconnected() {//ask, weather it is connected
  return connected;//are wo connected? So tell him
}

tcp_socket::tcp_socket() : base_socket() {//constructor of the tcp_socket, creates the descriptor
  block = true;//it is allowed to block
  listening = false;
}

tcp_socket::tcp_socket(bool _block) : base_socket() {
  block = _block;//is it allowed    if (ident -> socket != sockets[ident -> index]]) throw new socket_e(-1, "not placed"); to block?
  listening = false;
}

tcp_socket::~tcp_socket() {//destructor of the tcp_socket
  free(datawaiting);//remove the data that can be waiting
}

void tcp_socket::open(unsigned address, int port) {//so, connect to that wanted address
  preopen();//get the socket descriptor
  sockaddr_in addr;//the address struct
  addr.sin_family = AF_INET;//set the address family
  addr.sin_port = htons(port);//convert the port into network byte-order
  addr.sin_addr.s_addr = htonl(address);//convert the address as well
  memset(&addr.sin_zero, 0, sizeof addr.sin_zero);//set the rest to zeros
  //so ve set the address, the only thing to be done is the connecting itself
  open(&addr);//so connect to the address
}

void tcp_socket::open(void *addr) {//the real connecting function
  if (connect(sockethandle, (sockaddr *) addr, sizeof *(sockaddr *) addr)) {//we could not connect, there has been some error
    std::string errstr = "could not connect because: ";
    errstr += GET_ERROR;
    throw new socket_e(18, errstr, sockethandle);//and send the error up
  }//so, the connection worked
  connected = true;//we are connected
}

void tcp_socket::preopen() {//prepare to connect
  if (connected) //it is already connected
    throw new socket_e(1, "Socket already connected, can not connect twice", sockethandle);//so, tell it the caller
  if (!hassock) {//I don't know how could we have it, but maybe..
    sockethandle = socket(PF_INET, SOCK_STREAM, 0);//get sime inet stream socket
    if (sockethandle == -1) {//we did not get it ??
      std::string errstr = "could not get the socket descriptor becouse of error: ";
      errstr += GET_ERROR;//make the an error message
      throw new socket_e(2, errstr, sockethandle);//and send out the message we did not this
    }
    if (!block) set_non_block(sockethandle);//set it non blocking
  }
  hassock = true;//mark it as we have the socked allocated
} //no we should end up with valid socket descriptor

void tcp_socket::disconnect() {//disconnect from the host
  if (!connected) throw new socket_e(3, "socket not connected", sockethandle);//what do they want from me?
  CLOSE(sockethandle);//close the connection
  connected = false;//we are no more connected
  hassock = false;//by closing, the socket stops existing
  free(datawaiting);
  datawaiting = NULL;//reset the waiting data
  listening = false;
}

void tcp_socket::open(const std::string &hostname, int port) {//connect by host name and port
  preopen();//get the socket handle before connecting
  sockaddr_in addr;//the address struct
  addr.sin_family = AF_INET;//set the address family
  addr.sin_port = htons(port);//convert the port into network byte-order
  struct hostent *ipval = gethostbyname(hostname.c_str());//get the name
  if (!ipval) {
    std::string errstr = "could not loogup host: ";
    errstr += strerror(H_ERRNO);//get the string
    throw new socket_e(4, errstr);
  }
  if (ipval -> h_addrtype != AF_INET) {
    throw new socket_e(28, "unknown address family");
  }
  memcpy(&addr.sin_addr, ipval -> h_addr_list[0], 4);
  memset(&addr.sin_zero, 0, sizeof addr.sin_zero);//set the rest to zeros
  open(&addr);//and connect now
}

void tcp_socket::listenonport(int port) {//listen for connection
  if (connected) throw new socket_e(1, "connected socket can not listen", sockethandle);//it is already used
  preopen();
  sockaddr_in addr;//the address structure
  addr.sin_family = AF_INET;//the internet address family
  addr.sin_port = htons(port);//set the wanted port
  addr.sin_addr.s_addr = htonl(INADDR_ANY);//receive any address incoming
  memset(&addr.sin_zero, 0, sizeof addr.sin_zero);//set the rest to zeros
  if (bind(sockethandle, (sockaddr *) &addr, sizeof addr) == -1) {//so, set the port for the socket
    std::string errstr = "could not bind socket because: ";
    errstr += GET_ERROR;//set the error message
    throw new socket_e(5, errstr, sockethandle);//and send the message
  }//well, the socket is already bind, so now listen
  if (listen(sockethandle, 20) == -1) {//try to listen, if someone is calling me
    std::string errstr = "could not listen for incoming call because: ";
    errstr += GET_ERROR;//so, make the error message
    throw new socket_e(6, errstr, sockethandle);//and send the error
  }
  listening = true;
  connected = false;
}

void tcp_socket::listenonport(int port, unsigned allowaddr, unsigned char maxqueue) {//listen with some more setup
  if (connected) throw new socket_e(1, "connected socket can not listen", sockethandle);//it is already used
  preopen();//get the socket descriptor
  sockaddr_in addr;//the address structure
  addr.sin_family = AF_INET;//the internet address family
  addr.sin_port = htons(port);//set the wanted port
  addr.sin_addr.s_addr = htonl(allowaddr);//what address to allow?
  memset(&addr.sin_zero, 0, sizeof addr.sin_zero);//set the rest to zeros
  if (bind(sockethandle, (sockaddr *) &addr, sizeof addr) == -1) {//so, set the port for the socket
    std::string errstr = "could not bind socket because: ";
    errstr += GET_ERROR;//set the error message
    throw new socket_e(5, errstr, sockethandle);//and send the message
  }//well, the socket is already bind, so now listen
  if (listen(sockethandle, maxqueue) == -1) {//try to listen, if someone is calling me
    std::string errstr = "could not listen for incoming call because: ";
    errstr += GET_ERROR;//so, make the error message
    throw new socket_e(6, errstr, sockethandle);//and send the error
  }
  listening = true;
  connected = false;
}

void tcp_socket::write(const void *data, int size) {//put there some data
  if (!connected) throw new socket_e(6, "trying to send data trough unconnected tcp socket", sockethandle);
  if (datawaiting) throw new socket_e(7, "data waiting to be send/resend", sockethandle);
  int result = send(sockethandle, (char *)data, size, 0);//send the data adn see, how much it was actually sent
  if (result == size) return;//all went OK
  if (result == -1) {//some error occured
    std::string errstr = "could not send data because: ";
    errstr += GET_ERROR;//get the error message
    throw new socket_e(8, errstr, sockethandle);//and send the error
  }
  if (result < size) {
    datalen = size - result;//get the size
    if (!(datawaiting = malloc(datalen))) {//so, get space for the data
      throw new exception(0, "allocation error");//did not allocate the memory
    }
    memcpy(datawaiting, (char *) data + result, datalen);//copy the unsent rest
    datasent = 0;//how much of this is sent - nothing yet
    throw new socket_e(7, "data did not fit in one call, please call resend function", sockethandle);//so, tell that is too big
  }
  throw new socket_e(8, "??send more data than you gave me", sockethandle);
}

void tcp_socket::resend() {
  if (!connected) throw new socket_e(6, "trying to send data trough unconnected tcp socket", sockethandle);//you can't do that !!
  if (!datawaiting) throw new socket_e(9, "no data waiting to be sent", sockethandle);//there's nothing
  int result = send(sockethandle, (char *) datawaiting + datasent, datalen - datasent, 0);//send the rest now
  if (result == datalen - datasent) {//it was sent at last
    free(datawaiting);
    datawaiting = NULL;//there are no data waiting
  }
  if (result == -1) {
    std::string errstr = "could not send data because: ";
    errstr += GET_ERROR;//get the error message
    throw new socket_e(8, errstr, sockethandle);//and send the error
  }
  if (result < datalen - datasent) {//we did not sent all
    datasent += result;
    throw new socket_e(7, "resend again, not all sent", sockethandle);
  }
  throw new socket_e(8, "??send more data than you gave me", sockethandle);//this is really odd
}

void tcp_socket::resetwaiting() {
  free(datawaiting);
  datawaiting = NULL;//no data are waiting
}

bool tcp_socket::iswaiting() {
  if (datawaiting) return true; else return false;//return weather some data are waiting
}

void tcp_socket::addwaiting(const void *data, int size) {
  if (datawaiting && datasent) {//there are some data and some were sent
    int newlen = datalen - datasent + size;//how much data will we have after that?
    void *newdata = malloc(newlen);//get the space for it
    memcpy(newdata, (char *) datawaiting + datasent, datalen - datasent);//copy the rest of the previous one
    memcpy((char *) newdata + datalen - datasent, data, size);//get the new one there
    free(datawaiting);//now free the old one
    datawaiting = newdata;//and use the new one
    datasent = 0;//no data of the new one sent
    datalen = newlen;//and the length changed as well
  } else {
    if (!datawaiting) datalen = 0;//so, if there is no data..
    datawaiting = realloc(datawaiting, datalen + size);//get more space for new data
    memcpy((char *) datawaiting + datalen, data, size);//copy the data
    datalen += size;//mark the data block larger
  }
}

int tcp_socket::read(void *data, int max) {
  int result = recv(sockethandle, (char *) data, max, MSG_DONTWAIT);//read the data, non-blocking
  if (result == -1) {//some error occured
    if (ERRNO == EWOULDBLOCK) {//no data availible and the socket is non-blocking
      return 0;//no data read at all
    }
    std::string errstr = "could not read from socket becoause: ";
    errstr += GET_ERROR;//get the error
    throw new socket_e(10, errstr, sockethandle);//and sent the error up
  }
  if (result == 0) {//the peer closed the connection
    disconnect();//so we close it as well
    throw new socket_e(11, "connection closed by peer", sockethandle);//so tell the owner
  }
  return result;//return how much it read
}

int tcp_socket::readblocking(void *data, int max) {
  if (!block) throw new socket_e(13, "socket can not block", sockethandle);//the socket can not block
  int result = recv(sockethandle, (char *)data, max, 0);//read the data
  if (result == -1) {//some error
    std::string errstr = "could not read from socket becoause: ";
    errstr += GET_ERROR;//get the error
    throw new socket_e(10, errstr, sockethandle);//and sent the error up
  }
  if (result == 0) {//the other end closed it
    disconnect();//so we close it as well, it is no goot now
    throw new socket_e(11, "connection closed by peer", sockethandle);//telling the owner
  }
  return result;//so return how much of it was read
}

void tcp_socket::readexact(void *data, int size) {
  if (!block) throw new socket_e(13, "socket can not block", sockethandle);//the socket can not block
  int result = recv(sockethandle, (char *) data, size, MSG_WAITALL);//get all the data
  if (result == -1) {//some error
    std::string errstr = "could not read from socket becoause: ";
    errstr += GET_ERROR;//get the error
    throw new socket_e(10, errstr, sockethandle);//and sent the error up
  }
  if (result == 0) {//closed by peer
    disconnect();//close it
    throw new socket_e(11, "connection closed by peer", sockethandle);
  }
  if (result < size) {//for some reason, we did not get enough data
    throw new socket_e(12, "we did not get enough data", sockethandle);
  }
}

tcp_socket *tcp_socket::acceptconnection() {
  if (!listening) throw new socket_e(16, "not listening", sockethandle);//how you want to get connection
  int result = accept(sockethandle, NULL, 0);//don't get the IP address
  if (result == -1) {//some error on the way
    if (ERRNO == EWOULDBLOCK) {//this would block, however, it cant
      throw new socket_e(15, "no connection waiting", sockethandle);//so, this is quite special error
    }                                                                                    
    std::string errstr = "could not accept incoming connection because: ";
    errstr += GET_ERROR;//get the error
    throw new socket_e(14, errstr, sockethandle);//send the error
  }
  if (!block) set_non_block(sockethandle);//set it's blocking
  tcp_socket *newsock = new tcp_socket(block);//create new socket
  newsock -> sockethandle = result;//give it the socket
  newsock -> hassock = true;
  newsock -> connected = true;
  newsock -> listening = false;//set the flags
  return newsock;//and return the new socket that we accepted
}

tcp_socket *tcp_socket::acceptconnection(unsigned &_addr, int &port) {
  if (!listening) throw new socket_e(16, "not listening", sockethandle);//it is imposible to get connection with nobody listening
  sockaddr_in addr;//get the space for address
  socklen_t len = sizeof addr;
  int result = accept(sockethandle, (sockaddr *) &addr, &len);//accept the connection
  if (result == -1) {
    if (ERRNO == EWOULDBLOCK) {//this would block, however, it cant
      throw new socket_e(15, "no connection waiting", sockethandle);//so, this is quite special error
    }
    std::string errstr = "could not accept incoming connection because: ";
    errstr += GET_ERROR;//get the error
    throw new socket_e(14, errstr, sockethandle);//send the error
  }                                                         
  if (!block) set_non_block(sockethandle);//set it's blocking
  tcp_socket *newsock = new tcp_socket(block);//create new socket
  newsock -> sockethandle = result;//give it the socket
  newsock -> hassock = true;
  newsock -> connected = true;
  newsock -> listening = false;//set the flags
  _addr = ntohl(addr.sin_addr.s_addr);//get the address
  port = ntohs(addr.sin_port);//and the port
  return newsock;//and return the new socket that we accepted
}

server_socket::server_socket() {//constructor of the server socket
  to_delete = false;
}

server_socket::~server_socket() {//destructor
}

class simple_socket : public server_socket {
protected:
  friend class server;//let the server in
  int handle;//this is handle of the socket
  address remote;
  bool remove_data(int size);//remove first size bytes of data from queue and tell, weather it is already empty
  char *queue;
  int allocated;
  int size;//queue fro writing and its sizes
public:
  simple_socket();//constructor - no handle at all
  virtual ~simple_socket();//destructor
  virtual address getremoteaddr() const {return remote;}//return the address
  virtual int gethandle() const {return handle;}//get the handle
  virtual void write(int size, const char *data);//write data to the queue
  virtual bool queue_empty() const;//anwser, weather it is empty
  virtual int queue_size() const {return size;}//just return the size
};//this is virtual between - step for managing network sockets

simple_socket::simple_socket() {//construcotr of simple socket
  handle = 0;//no socket
  queue = NULL;
  size = 0;
  allocated = 0;//no queue at the moment
}

simple_socket::~simple_socket() {
  free(queue);//free the queue-space
  if (handle) CLOSE(handle);
}

void simple_socket::write(int _size, const char *data) {//put it into queue and wait for it to be sent
  if (size + _size > allocated) queue = (char *) realloc(queue, allocated += _size * 2);//is it large enough?
  memcpy(queue + size, data, _size);//copy the data
  size += _size;//increase the size by the new data
}

bool simple_socket::queue_empty() const {
  return (size == 0);//try, weather it is empty
}

bool simple_socket::remove_data(int _size) {
  memmove(queue, queue + _size, size -= _size);//movethe data towards beginig
  if (!size) {
    free(queue);
    queue = NULL;
    size = 0;
    allocated = 0;//free it all, we sent it
    return true;//and it is EMPTY
  }
  if (allocated / 3 > size) queue = (char *) realloc(queue, allocated = size);//shrink it so it will fit
  return false;//there still is something
}

int get_tcp_socket(address_family af) {//generates non-blocking tcp socket
  int result;//there will be the socket for a while
#ifndef VORLIB_NO_IPV6 //does we support that
  if (af == af_ipv6) result = socket(PF_INET6, SOCK_STREAM, 0);//create ipv6 socket
  else
#endif
  result = socket(PF_INET, SOCK_STREAM, 0);//create standart inet socket
  if (result == -1) {//so, there was some error
    std::string errstr = "could not get socket handle: ";
    errstr += GET_ERROR;//get the error message
    throw new socket_e(2, errstr);//and send the error above
  }
  set_non_block(result);//the socket will never block
  return result;//return the socket
}

class tcp_listen_socket : public simple_socket {
public:
  tcp_listen_socket(const address &addr, unsigned char maxqueue);//create and listen
  virtual server_socket_type gettype() const {return sst_tcp_listen;};//get the type of socket
  virtual address getremoteaddr() const {throw new socket_e(16, "listening port has no remote end", handle);}//just argue with him
  virtual bool wantwrite() const {return false;}//well, listening socket never writes, it is like that
  virtual bool canread() const {return true;}//I listen all time, so I will be able to be read when someone calls me
  virtual void write(int, const char *) {throw new socket_e(-1, "listen socket ca not write", handle);}
  virtual bool socket_ready() {throw new socket_e(-1, "listen socket tolt to write", handle);}//it is insane
  virtual bool queue_empty() const {throw new socket_e(-1, "no queue within listen socket", handle);}
  virtual int queue_size() const {throw new socket_e(-1, "no queue within listen socket", handle);}
};//class for listening on tcp

tcp_listen_socket::tcp_listen_socket(const address &addr, unsigned char maxqueue) : simple_socket() {
  handle = get_tcp_socket(addr.get_af());//get the socket
  int result = bind(handle, (sockaddr *) addr.get_addr(), sizeof *addr.get_addr());//bind it there
  if (result == -1) {
    std::string errstr = "could not bind socket because: ";
    errstr += GET_ERROR;//create the message string
    throw new socket_e(5, errstr, handle);//and throw the error
  }
  result = listen(handle, maxqueue);//start listening
  if (result == -1) {
    std::string errstr = "could not listen because: ";
    errstr += GET_ERROR;//get the message
    throw new socket_e(6, errstr, handle);//and throw it
  }
  //now, we are listening for incoming connection
}

class tcp_transfer_socket : public simple_socket {
protected:
  friend class server;//let the server in, so it can see the connected
  bool connected;//is it alreaddy connected?
public:
  tcp_transfer_socket(const address &addr);//tconstructor that connects to remote host
  tcp_transfer_socket(int socket);//just take this socket as connected (from accept, for example)
  virtual ~tcp_transfer_socket();//destructor
  virtual server_socket_type gettype() const;//get the type of the socket
  virtual bool wantwrite() const;//Do I want to send something?
  virtual bool canread() const {return connected;};//I can, when I'm connected
  virtual bool socket_ready();//send some data trough
};//class for the tcp socket that actually transfers some data

tcp_transfer_socket::tcp_transfer_socket(const address &addr) : simple_socket() {
  handle = get_tcp_socket(addr.get_af());//get the handle
  connected = false;//we are not connected, we just begin
  remote = addr;//set the  address
  if (connect(handle, (sockaddr *) addr.get_addr(), sizeof *(addr.get_addr())) == -1) {//connect there, it will tell us when it is ready
    if (ERRNO == EINPROGRESS) {//it is OK, just wait for connection
      return;//just wait for it to finish
    }
    //now, it seems to be a real problem
    std::string errstr = "could not connect because: ";
    errstr += GET_ERROR;//just get the error code
    throw new socket_e(18, errstr, handle);//send the error away
  }
  connected = true;//wow, it did not block and made it imediatelly, it's nice
}

tcp_transfer_socket::~ tcp_transfer_socket() {
}

tcp_transfer_socket::tcp_transfer_socket(int socket) : simple_socket () {
  handle = socket;//set the handle for the new one
  connected = true;//we got it already connected
}

bool tcp_transfer_socket::wantwrite() const {
  if (connected) return !queue_empty();//is there something I want to send?
  else return true;//Yes, actually, I want to connect..
}

bool tcp_transfer_socket::socket_ready() {
  if (connected) {//so, we want to send some data
    int res = send(handle, queue, size, 0);//for the begining, try to send whole queue
    if (res == 0) {
      throw new server_internal_e(1);
    }
    if (res == -1) {//some error
      std::string errstr = "error sending data: ";
      errstr += GET_ERROR;//so, make up some error message
      throw new socket_e(8, errstr, handle);//send the error out
    }
    return remove_data(res);//remove them from queue
  } else {//it means that we are connected, hurray
    int err;
    socklen_t esize = sizeof err;
#ifdef VORLIB_UNIX
    if (getsockopt(handle, SOL_SOCKET, SO_ERROR, &err, &esize) == -1) {//I could not read the status
#endif
#ifdef VORLIB_WINDOWS
    if (getsockopt(handle, SOL_SOCKET, SO_ERROR, (char *) &err, &esize) == -1) {//I could not read the status
#endif
      std::string errstr = "could not read socket options: ";
      errstr += GET_ERROR;
      throw new socket_e(25, errstr, handle);
    }
    if (err) {//there was some error connecting
      std::string errstr = "fatal error while connecting: ";
      errstr += strerror(err);
      SETERROR(err);
      throw new socket_e(26, errstr, handle);
    }
    connected = true;//we are connected
    return (size == 0);//well, it should be empty, but who knows
  }
}

server_socket_type tcp_transfer_socket::gettype() const {
  if (connected) return sst_tcp_transfer; else return sst_tcp_waiting_connect;//tell him, what are you 
}

class tcp_line_socket : public tcp_transfer_socket {
private:
  char *line;
  int linesize;
  int linealloc;
  char *line_ready;//this line is ready here
public:
  tcp_line_socket(int socket);//constructor with already existing socket
  tcp_line_socket(const address &addr);//connecting constructor
  virtual ~tcp_line_socket();//destructor
  virtual server_socket_type gettype() const;//return my type
  void put_data(const void *data, int data_size);//put some read data into line-queue, it will be taken out soon
  char *prepare_line(int &length);//gets line from the queue, removes it from there, returns a string NUL terminated. It needs to be freed after that. length is length of that string (without the NUL)
};

tcp_line_socket::tcp_line_socket(const address &addr) : tcp_transfer_socket(addr) {
  line = NULL;//the line is empty now
  linesize = 0;//and its size is therefore emptty
  linealloc = 0;//and nothing allocated
}

tcp_line_socket::tcp_line_socket(int socket) : tcp_transfer_socket(socket) {
  line = NULL;
  linesize = 0;
  linealloc = 0;
}

tcp_line_socket::~tcp_line_socket() {
  free(line);//dealloc that (if there is anything)
}

server_socket_type tcp_line_socket::gettype() const {
  if (connected) return sst_line_transfer; else return sst_tcp_waiting_connect;//are we connected?
}

void tcp_line_socket::put_data(const void *data, int data_size) {
  if (linealloc < linesize + data_size) //there is not enough space
   line = (char *) realloc(line, linealloc += (linesize + data_size - linealloc) * 2);
   //get more space for data
  memcpy(line + linesize, data, data_size);//copy the data
  linesize += data_size;//and say it's bigger
}

char *tcp_line_socket::prepare_line(int &len) {
  char *ec = line + linesize;//get one after the last so it wont run over
  bool newline = false;//we did not found the new line character
  char *pos = NULL;
  for (char *c = line; c != ec; c++) {
    if ((*c == '\n') || (*c == '\r')) newline = true;
    else if (newline) {
      pos = c;//it is the last before new line context
      break;//I already found it, there is no need to keep looking
    }
  }
  if (pos) {//it was found and there is something after that
    int length = pos - line;//how long is the part?
    char *result = (char *) malloc(length + 1);//get space to copy it
    memcpy(result, line, length);//copy the line
    result[length] = '\0';//set the last one
    memmove(line, line + length, linesize - length);//move the rest to the begining
    if ((linesize -= length) <= linealloc / 2) line = (char *) realloc(line, (linealloc = linesize));//shrink it if it's large
    len = length;//set the line length there
    return result;//return it
  }
  if (newline) {//there is the newline and there is nothing more after that
    len = linesize;//set the length
    if (linesize == linealloc) line = (char *) realloc(line, linesize + 1);//the trailing NUL char would not fit there
    line[linesize] = '\0';//set the bumper there
    char *tmp = line;//put it there
    line = NULL;//it wont be there
    linealloc = 0;
    linesize = 0;
    return tmp;
  }
  return NULL;//nothing was found
}

class console_socket : public server_socket {
private:
public:
  console_socket();//constructor
  virtual server_socket_type gettype() const {return sst_console;};//this is console socket all the time
  virtual address getremoteaddr() const {throw new socket_e(16, "console has no remote end", 0);}//well, this is as stupid as the one before
  virtual int gethandle() const {return 0;}//console is on the socket 0
  virtual bool wantwrite() const {return false;}//actually, you can write to stdout all the time, so there is no need to watch it
  virtual bool canread() const {return true;}//well, why else I would watch the console if I didn't want to read it?
  virtual void write(int, const char *) {throw new socket_e(23, "Can not write to console, write manually", 0);}
  virtual bool socket_ready() {throw new socket_e(-1, "console told to write", 0);}
  virtual bool queue_empty() const {throw new socket_e(-1, "console has no write queue", 0);}
  virtual int queue_size() const {throw new socket_e(-1, "console has no write queue", 0);}
};

console_socket::console_socket() : server_socket() {
}

class console_line_socket : public server_socket {
private:
  char *buffer;//buffer for reading
  unsigned buffsize;//
  std::string line;//here will be the line
public:
  console_line_socket();//constructor
  ~console_line_socket();//destructor
  server_socket_type gettype() const {return sst_line_console;}
  address getremoteaddr() const {throw new socket_e(16, "console has no remote address", 0);}
  int gethandle() const {return 0;}//console is on 0
  bool wantwrite() const {return false;}//well, you can write there as much as you want, so just I don't check it
  bool canread() const {return true;}//actually, I want to read from the console, it is because it is here
  void read_console();//read into buffer
  bool line_ready();//prepare the line to be send out
  std::string get_line();//get the prepaered line
  virtual void write(int, const char *) {throw new socket_e(23, "Can not write to console, write manually", 0);}
  virtual bool socket_ready() {throw new socket_e(-1, "console told to write", 0);}
  virtual bool queue_empty() const {throw new socket_e(-1, "console has no write queue", 0);}
  virtual int queue_size() const {throw new socket_e(-1, "console has no write queue", 0);}
};

console_line_socket::console_line_socket() : server_socket() {
  buffer = NULL;//there is nothing yet
  buffsize = 0;//it is empty
}

console_line_socket::~console_line_socket() {
  free(buffer);//free the buffer
}

void console_line_socket::read_console() {//the console is ready, I will read it all and put it into buffer
#ifdef VORLIB_UNIX
  char buff[100];//read buffer
  int size = read(0, buff, 100);//read 100 chars max from input
  if (size == -1) {
    std::string errstr = "console reading error: ";
    errstr += GET_ERROR;//make the message
    throw new socket_e(21, errstr, 0);//and throw the error
  }
  unsigned len = (buffer) ? strlen(buffer) : 0;
  if (len + 1 + (unsigned) size > buffsize) buffer = (char *) realloc(buffer, (buffsize = len + 1 + size));//get space for the buffer
  memcpy(buffer + len, buff, size);//copy the new part into buffer
  buffer[len + size] = 0;//end the string
#endif //!!THERE IS NO VERSION FOR WINDOWS, THEY CAN'T DO THIS AT ALL 
}

 bool console_line_socket::line_ready() {
  if (!buffer) return false;//there is nothing
  line = "";//the line is empty at the begining
  for (char *c = buffer; *c; c++) {
    switch (*c) {
      case '\n': {//newline
        memmove(buffer, c + 1, strlen(c));//copy the rest rest
        if (!*(c + 1)) {//it is empty, so free it's memory
          free(buffer);
          buffer = NULL;
          buffsize = 0;
        } else
        if (strlen(buffer) + 100 < buffsize) buffer = (char *) realloc(buffer, (buffsize = strlen(buffer) + 1));//shrink it
        return true;//we made the line, we have enough
      }
      default: {
        line += *c;//add it there
        break;
      }
    }
  }
  return false;//it ended and no new line found
}

std::string console_line_socket::get_line() {
  return line;//return the line
}

typedef struct {
  long_sockaddr addr;//the destination address
  int size;//how large it is
  void *data;//and the data itself
} udp_packet;

class udp_socket : public server_socket {
private:
  address_family af;
  bool last;//do we have last port and address?
  address source;//the last source address
  int handle;//handle of that socket
  address dest;//the destination
  int queuesize;
  int queuealloc;//how big is the queue - how many packets
  int datasize;//how many bytes are in the queue
  udp_packet *queue;//the queue for data
public:
  udp_socket(address_family af);//constructor
  ~udp_socket();//destructor
  virtual server_socket_type gettype() const {return sst_udp;};//get its type
  virtual address getremoteaddr() const {return dest;};//the destination address in thsi case
  virtual int gethandle() const {return handle;};//the handle of socket
  virtual bool wantwrite() const {return (queuesize != 0);};//do I want to write something?
  virtual bool canread() const {return true;};//it can read always
  virtual void write(int size, const char *data);//add some data into the queue
  virtual bool socket_ready();//we can write. return, weather the queue is empty
  virtual bool queue_empty() const {return (queuesize == 0);};//there aren't data waiting, are they
  virtual int queue_size() const {return datasize;};//return count of bytes waiting
  void set_local(int port);//sets local port
  void set_remote(const address &addr);//sets the destination
  const address &get_source_addr() const;//gets the address of the last  incoming packet
  void set_source(const address &addr);//just stores the last source address there
};//the udp socket for server

int get_udp_socket(address_family af) {
  int result;//there will be the descriptor
#ifndef VORLIB_NO_IPV6 //are we supporting the ipv6 this compile?
  if (af == af_ipv6) result = socket(PF_INET6, SOCK_DGRAM, 0); else//create the ipv6 socket
#endif
  result = socket(PF_INET, SOCK_DGRAM, 0);//so, ipv4 socket
  if (result == -1) {//oops, some error
    std::string errstr = "could not get the socket descriptor: ";
    errstr += GET_ERROR;//get the error string
    throw new socket_e(2, errstr);//throw the error
  }
  set_non_block(result);//the socket will never block
  return result;
}

udp_socket::udp_socket(address_family af) {
  handle = get_udp_socket(af);//create the descriptor
  last = false;//wi did not get any data, we just created the socket
  queuesize = 0;
  queuealloc = 0;//the queue is empty
  datasize = 0;//no data to send
  queue = NULL;//no queue
  this -> af = af;
}

udp_socket::~udp_socket() {
  for (int i = 0; i < queuesize; i++) {
    free(queue[i].data);//free data in all unsent packets
  }
  free(queue);//rmove the array as well
  CLOSE(handle);//remove the socket
}

void udp_socket::write(int size, const char *data) {
  if (queuealloc < ++queuesize) queue = (udp_packet *) realloc(queue, (queuealloc += 20) * sizeof *queue);//make it lareger, if it is needed
  udp_packet *pack = queue + queuesize - 1;//get the actual one into some shorter var
  memcpy(&pack -> addr, dest.get_addr(), sizeof (dest.get_addr()));
  pack -> size = size;//how big is it?
  pack -> data = malloc(size);//create place for that
  memcpy(pack -> data, data, size);//copy the data
  datasize += size;
}

bool udp_socket::socket_ready() {
  int pos = 0;//witch packet we are on?
  int result;//the result of calling
  int sent = 0;//it contains number of packets we sent ok
  do {
    result = sendto(handle, (char *) queue[pos].data, queue[pos].size, 0, (sockaddr *) &queue[pos].addr, sizeof queue[pos].addr);
    if (result > 0) {
      sent ++;//it is sent
      datasize -= queue[pos].size;
    }
    pos ++;
  } while ((pos < queuesize)  && (result > 0));
  for (int i = 0; i < sent; i++)
    free(queue[i].data);//free all the data we sent out
  memmove(queue, queue + sent, sent * sizeof *queue);//move them a bit to the begining, so the sent will be replaced
  queuesize -= sent;//there are less packets now
  if (queuesize * 2 < queuealloc) queue = (udp_packet *) realloc(queue, (queuealloc = queuesize) * sizeof *queue);//shrink it, if it is too large  
  if (result > 0) return true;//the queue is empty, we just sent all
  if (ERRNO == EWOULDBLOCK) {
    return false;//there are still some data
  }
  std::string errstr = "send error: ";
  errstr += GET_ERROR;
  throw new socket_e(8, errstr, handle);
}

void udp_socket::set_local(int port) {
  long_sockaddr addr;//the address space
  memset(addr.data, 0, sizeof addr.data);//set it to 0
#ifndef VORLIB_NO_IPV6
  if (af == af_ipv6) addr.af = AF_INET6;//set the address family
  else
#endif
  addr.af = AF_INET;
  ((ipv4_addr *) &addr) -> port = htons(port);//the port is on the same place for both of them
  if (bind(handle, (sockaddr *) &addr, sizeof addr) == -1) {//bind it
    std::string errstr = "could not bind socket because: ";
    errstr += GET_ERROR;//create the error message
    throw new socket_e(5, errstr, handle);//throw the error
  }
}

void udp_socket::set_remote(const address &addr) {
  dest = addr;//set the address
}

const address &udp_socket::get_source_addr() const {
  if (last) return source;//just give it him
  else throw new socket_e(29, "No source address yet", handle);
}

void udp_socket::set_source(const address &addr) {
  last = true;
  source = addr;
}

server::server() {//constructor of tcp line server
  socketcount = 0;
  prealloc = 0;//nothing allocated
  sockets = NULL;//no sockets at the begining
  timer_interval = 30;//set default interval
  timer_event = NULL;//no timer at the begining
  error_event = NULL;//do nothing on errors by default
  console_ready_event = NULL;
  console_line_event = NULL;
  tcp_listen_event = NULL;
  new_socket_event = NULL;
  del_socket_event = NULL;
  data_arrive_event = NULL;
  queue_empty_event = NULL;
  data_sent_event = NULL;
}

server::~server() {
  server_socket **sock_e = sockets + socketcount;//get that one behind the last one
  for (server_socket **sock = sockets; sock != sock_e; sock++) //run trough all axisting sockets
    delete *sock;//and delete them all
  free(sockets);//free the array
}

server::server(const server &) {
  throw new socket_e(17, "You can not copy the server");//you just can't do that
}

void server::operator=(const server &) {
  throw new socket_e(17, "You can not copy the server");
}

int server::addsock(server_socket *sock) {//add some socket to the list
  if (socketcount == prealloc) {//is it large enough?
    sockets = (server_socket **) realloc(sockets, (prealloc += 10) * sizeof *sockets);//resize the array
  }
  sockets[socketcount++] = sock;//and place the new one there
  socket_ident ident = {sock, socketcount - 1};//create the socket identificator
  indexer.add(sock -> gethandle(), ident);//add it to indexer
  if (new_socket_event) new_socket_event(sock -> gethandle(), sock -> gettype(), this);//tell the user we added a new socket
#ifdef _DEBUG
  checkall();
#endif
  return sock -> gethandle();//return the sockets handle
}

int server::tcp_listen(int port, address_family af) {
  long_sockaddr addr;//there will be composed the address
#ifndef VORLIB_NO_IPV6
  if (af == af_ipv6) {
    addr.af = AF_INET6;//it is the inet address family, version 6
  }
#endif
  {
    addr.af = AF_INET;//the standart v4 version
  }
  memset(addr.data, 0, sizeof addr.data);//fill it all with zeros
  ((ipv4_addr *) &addr) -> port = htons(port);//set the port to listen to
  return addsock(new tcp_listen_socket(getaddr(&addr), 10));//just add listening socket
}

int server::tcp_listen(const address &addr, unsigned char maxqueue) {
  return addsock(new tcp_listen_socket(addr, maxqueue));//new listening socket by address and so
}

int server::tcp_connect(const address &addr) {
  int result = addsock(new tcp_transfer_socket(addr));//add the socket
  if (((tcp_transfer_socket *) sockets[socketcount - 1]) -> connected && queue_empty_event) queue_empty_event(result, this);//actually, I KNOW it is the last one, I just added it there
  return result;
}

int server::line_connect(const address &addr) {
  int result = addsock(new tcp_line_socket(addr));//add the socket
  if (((tcp_line_socket *) sockets[socketcount - 1]) -> connected && queue_empty_event) queue_empty_event(result, this);
  return result;
}

int server::add_console() {
  if (indexer % 0) throw new socket_e(20, "you already scan the console");//the console should be there only once
  return addsock(new console_socket());//add simple console socket
}

int server::add_line_console() {
  if (indexer % 0) throw new socket_e(20, "you already scan the console");//the console should be there only once
  return addsock(new console_line_socket());//add simple console socket
}

void server::terminate() {
  running = false;//stop the server
}

void server::go(int refresh) {
  running = true;
  while (running) {
    rebuild_lists();//make lists of them now
    timeval tv;//this is the timeval structure - how long it will select
    tv.tv_sec = 0;
    tv.tv_usec = 0;//we do not wait a bit
    fd_set r = readsocks;//copy them
    fd_set w = writesocks;//copy them too
    int result;
    try {
      do {
        readsocks = r;
        writesocks = w;//so, make a new copy of the full list
        if ((result = select(maxfd + 1, &readsocks, &writesocks, NULL, &tv)) == -1) {//wait, if something happens
          //oops, something ugly happened
          std::string errstr = "error while selecting: ";
          errstr += GET_ERROR;//just make a message
          throw new socket_e(19, errstr);//just throw the error
        }
        tv.tv_sec = refresh;//wait one second for the next try
        tv.tv_usec = 0;//exactly one second
      } while (!checkevents(result));//just keep selecting until something happens, then just make new queue and start again
    }
    catch (socket_e *e) {//well, some error? so, tell the owner
      raiseexcp(e);//this will take care of it
    }
  }
  delete_all();//delete all marked socket now
}

void server::rebuild_lists() {//get list of them all
  FD_ZERO(&readsocks);//clear the read sockets
  FD_ZERO(&writesocks);//clear the write sockets
  maxfd = 0;//nothing used yet, so the highest is 0
  server_socket **sock_e = sockets + socketcount;//get the one behind the last
  for (server_socket **sock = sockets; sock != sock_e; sock++) {
  if ((*sock) -> gethandle() > maxfd) maxfd = (*sock) -> gethandle();//is it larger? looking for the biggest
    if ((*sock) -> wantwrite()) FD_SET((*sock) -> gethandle(), &writesocks);//does it want to write now?
    if ((*sock) -> canread()) FD_SET((*sock) -> gethandle(), &readsocks);//can it read
  }
}

void server::set_onerror(errorevent e) {
  error_event = e;//set the value there
}

errorevent server::get_onerror() const {
  return error_event;//get the value of error event
}

void server::raiseexcp(socket_e *e) {
  if (error_event) {
    try {
      error_event(e, this);//call the event
    }
    catch (...) {}//actually, just ignore all exceptions, I said him not to use them here, it woudn't be nice, if it went on
    delete e;//and delete the exception, keep running
  } else throw e;//just let it go up..
}

void server::set_ontimer(timerevent e) {
  timer_event = e;//set the value
  reset_timer();//generate new timer now
}

timerevent server::get_ontimer() const {
  return timer_event;//just give him this
}

void server::set_interval(int i) {
  timer_interval = i;//set the interval
  reset_timer();//and reset the timer to make effect
}

int server::get_interval() const {
  return timer_interval;
}

void server::reset_timer() {
  time_next = time(NULL) + timer_interval;//set the next time, when the timer ticks
}

void server::set_onconsoleready(consolereadyevent e) {
  console_ready_event = e;
}

consolereadyevent server::get_onconsoleready() const {
  return console_ready_event;
}

void server::set_onconsoleline(consolelineevent e) {
  console_line_event = e;//set the event
}

consolelineevent server::get_onconsoleline() const {
  return console_line_event;
}

void server::set_ontcplisten(tcplistenevent e) {
  tcp_listen_event = e;
}

tcplistenevent server::get_ontcplisten() const {
  return tcp_listen_event;
}

void server::set_onnewsocket(newsocketevent e) {
  new_socket_event = e;
}

newsocketevent server::get_onnewsocket() const {
  return new_socket_event;
}

void server::set_ondelsocket(delsocketevent e) {
  del_socket_event = e;
}

delsocketevent server::get_ondelsocket() const {
  return del_socket_event;
}

void server::set_ondataarrive(dataarriveevent e) {
  data_arrive_event = e;
}

dataarriveevent server::get_ondataarrive() const {
  return data_arrive_event;
}

void server::delsock(int socket, delsocket_reason reason) {
  try {
    indexer[socket].socket -> to_delete = true;//mark him to be deleted
    if (del_socket_event) del_socket_event(socket, get_type(socket), reason, this);//tell the user that it is gouing away now
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);
    }
    throw e;
  }
}

bool server::checkevents(int count) {

#define RET(result) {delete_all(); return (result);}
  bool eventfound = false;
  if (timer_event && (time(NULL) >= time_next)) {//the timer has expired
    timer_event(this);//call the event
    reset_timer();//and set the new time for next call
    eventfound = true;//this is a legal event as well
  }
  if (count) {
    for (int i = 0; i < socketcount; i++) {
      server_socket *sock = sockets[i];//get the socket
      if (FD_ISSET(sock -> gethandle(), &readsocks)) {//is it ready to be read?
        try {
          switch (sock -> gettype()) {//well, it is different by the type how that works
            case sst_console: {
              if (console_ready_event) console_ready_event(this);//just tell the user that the console is ready for reading
              break;
            }
            case sst_line_console: {
              if (!console_line_event) break;//well, nobody listens, so why would I bother myself
              ((console_line_socket *) sock) -> read_console();//read the line
              while (((console_line_socket *) sock) -> line_ready())
                console_line_event(((console_line_socket *) sock) -> get_line(), this);//give the user all lines here
              break;
            }
            case sst_tcp_listen: {//there is a connection waiting
              if (!tcp_listen_event) {//nobody wants that
                int s = accept(sock -> gethandle(), NULL, NULL);//accept the connection and don't care about the remote address
                if (s == -1) {
                  std::string errstr = "error accepting connection: ";
                  errstr += GET_ERROR;//get the error message
                  throw new socket_e(14, errstr, sock -> gethandle());//throw the error
                }
                send(s, "REJECTED\n", 9, 0);//send the message to the remote host
                CLOSE(s);//and close it, don't wait for it to reply, don't care about it any more
              } else {//well, the ont tcp listen
                long_sockaddr addr;//space for the address
                socklen_t len = sizeof addr;
                int s = accept(sock -> gethandle(), (sockaddr *) &addr, &len);//accept the connection adn get address
                if (s == -1) {//some error occured
                  std::string errstr = "errror accepting connection: ";
                  errstr += GET_ERROR;//get the error message
                  throw new socket_e(14, errstr, sock -> gethandle());//and send it out
                }
                server_socket_type result = tcp_listen_event(getaddr(&addr), sock -> gethandle(), s, this);//ask user what to do with it
                switch (result) {
                  case sst_none: {//the useer does not like thisconnection
                    send(s, "REJECTED\n", 9, 0);
                    CLOSE(s);//tell the remote one he is not wanted and close him
                    break;
                  }
                  case sst_tcp_transfer: {//create simple tcp transfer socket
                    int news = addsock(new tcp_transfer_socket(s));//we already have the socket
                    simple_socket *newsock = (simple_socket *) sockets[socketcount - 1];//get the new socket
                    newsock -> remote = getaddr(&addr);//copy the  address
                    if (queue_empty_event) queue_empty_event(news, this);//the empty event
                    break;
                  }
                  case sst_line_transfer: {//create new line socket
                    int news = addsock(new tcp_line_socket(s));//just create new socket with this socket
                    simple_socket *newsock = (simple_socket *) sockets[socketcount - 1];//get the last one - its the one I just created
                    newsock -> remote = getaddr(&addr);
                    if (queue_empty_event) queue_empty_event(news, this);//this socket must have empty queue
                    break;//and quit processing
                  }
                  default: {
                    throw new socket_e(23, "can not create this type of socket at accept", s);
                  }
                }
              }
              break;
            }
            case sst_tcp_transfer: {//the socket is ready to be read (or something else, who knows)
              int s = sock -> gethandle();//get the handle into some shorter var
              char buff[10240];//the buffer for reading, if it is not enough, it will be read next time
              int result = recv(s, buff, 10240, 0);//read max 10K of data into buffer
              switch (result) {
                case -1: {//oups, error
                  std::string errstr = "error reading from tcp socket: ";
                  errstr += GET_ERROR;//get the error
                  if ((ERRNO == ECONNRESET) || (ERRNO == ESHUTDOWN) || (ERRNO == ECONNABORTED)) throw new socket_e(24, errstr, s);//it is reset by peer error
                  throw new socket_e(10, errstr, s);//throw the error
                }
                case 0: {//the socket closed
                  delsock(s, dsr_remote_closed);//the remote one closed it
                  break;//so, we have done it
                }
                default: {//we actualy read something
                  if (data_arrive_event) data_arrive_event(s, buff, result, this);//so, give the user that data
                  break;//and stop it
                }
              }
              break;
            }
            case sst_line_transfer: {//this is line - managed
              int s = sock -> gethandle();//read the handle into some shorter var name
	      char buff[1024];//buffer for incoming data
	      int result = recv(s, buff, 10240, 0);//read 10k of data from socket, there won't be more probably and it would tell us again
	      switch (result) {
	        case -1: {//some error
		  std::string errstr = "error reading from tcp socket: ";
                  errstr += GET_ERROR;//get the error
                  if ((ERRNO == ECONNRESET) || (ERRNO == ESHUTDOWN) || (ERRNO == ECONNABORTED)) throw new socket_e(24, errstr, s);//it is reset by peer error
                  throw new socket_e(10, errstr, s);//throw the error
		}
		case 0: {//socket closed
		  delsock(s, dsr_remote_closed);//so remove our end
		  break;//nothing to be done with it any more
		}
		default: {//some data actually arrived
		  ((tcp_line_socket *) sock) -> put_data(buff, result);//give it its data
		  char *line;
		  int length;
		  while ((line = ((tcp_line_socket *) sock) -> prepare_line(length))) {
		    if (data_arrive_event) data_arrive_event(s, line, length, this);//give him the data if he wants them
		    free(line);//free the line, it is not needed
		  }
		}
	      }
	      break;//it is done
            }
            case sst_udp: {//the udp socket
              char buffer[65536];//64k is the maximum size of the udp packet
              int result;
              do {
                long_sockaddr addr;
                socklen_t len = sizeof addr;
                result = recvfrom(sock -> gethandle(), buffer, 65536, 0, (sockaddr *) &addr, &len);//read the packet and get its source address
                if (result == 0) {//socket closed - well, I think it makes no sence but, anyway
                  delsock(sock -> gethandle(), dsr_remote_closed);//so, remove it
                  break;//no more reading here, it is closed
                }
                if (result > 0) {
                  ((udp_socket *) sock) -> set_source(getaddr(&addr, len));//set the address there
                  if (data_arrive_event) data_arrive_event(sock -> gethandle(), buffer, result, this);//call the event
                }
              } while (result != -1);//do as much as you can
              if (ERRNO != EWOULDBLOCK) {//it is erron, not that there is no more data
                std::string errstr = "error reading from socket: ";
                errstr += GET_ERROR;
                throw new socket_e(10, errstr, sock -> gethandle());
              }
              break;
            }
            default: {
              throw new socket_e(-1, "this should not want to be read", sock -> gethandle());
            }
          }
        }
        catch (socket_e *e) {
          raiseexcp(e);//let him to handle the exception
          if (!running) RET(true);
        }
      }
      if (!sock -> to_delete) {//the socket still exists
        if (FD_ISSET(sock -> gethandle(), &writesocks)) {//So, write some data
          try {
            bool empty = sock -> socket_ready();//write there some data
            if (data_sent_event) data_sent_event(sock -> gethandle(), sock -> queue_size(), this);//we sent some data
            if (empty && queue_empty_event) queue_empty_event(sock -> gethandle(), this);
          }
          catch (server_internal_e *e) {
            if (e -> id == 1) {
              delsock(sock -> gethandle(), dsr_remote_closed);
            }
          }
          catch (socket_e *e) {
	    int id = e -> id;
            raiseexcp(e);//so, handle the exception
            if (id == 26) delsock(sock -> gethandle(), dsr_error);//removing that socket due to error
            if (!running) RET(true);//so, the owner does not want it to continue
          }
        }
      }
    }
    eventfound = true;//there definitelly is some event
  }
  RET(eventfound);//did we found an event?
#undef RET
}

server_socket_type server::get_type(int socket) const {
  try {
    return indexer[socket].socket -> gettype();//get the type of that socket
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);//tell him it is not a socket
    }
    throw e;//it is something else, just send it away
  }
}

address server::get_remote_addr(int socket) const {
  try {
    return indexer[socket].socket -> getremoteaddr();//get the type of that socket
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);//tell him it is not a socket
    }
    throw e;//it is something else, just send it away
  }
}

int server::get_sock_count() const {
  return socketcount;//return count of sockets
}

int server::get_socket(int index) const {
  return sockets[index] -> gethandle();//return handle of wanted socket
}

int server::operator[] (int index) const {
  return sockets[index] -> gethandle();//handle of that socket
}

void server::checkall() const {
  if (prealloc - 10 >= socketcount) throw new socket_e(-1, "too big prealloc");
  if (prealloc < socketcount) throw new socket_e(-1, "too small prealloc");
  if (socketcount < 0) throw new socket_e(-1, "socket count negative");
  if (!indexer.check()) throw new socket_e(-1, "tree fault");
  if (!indexer.depthtest()) throw new socket_e(-1, "tree depth fault");
  server_socket **sock_e = sockets + socketcount;
  for (server_socket **sock = sockets; sock != sock_e; sock++) {
    if (!(indexer % (*sock) -> gethandle())) throw new socket_e(-1, "not in tree");
  }
  std::vector<int> indexes = indexer.get_indexes();//get all indexes in the tree
  std::vector<socket_ident> idents = indexer.get_data();//get all idents
  std::vector<int>::iterator index = indexes.begin();//get first index
  for (std::vector<socket_ident>::iterator ident = idents.begin(); ident != idents.end(); ident++, index++) {
    if (ident -> index >= socketcount) throw new socket_e(-1, "out of bounds");
    if (ident -> socket != sockets[ident -> index]) throw new socket_e(-1, "not placed");
    if (*index != ident -> socket -> gethandle()) throw new socket_e(-1, "non-identical socket");
  }
}

bool server::queue_empty(int socket) const {
  try {
    return indexer[socket].socket -> queue_empty();//ask that socket
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);//this is not my socket
    }
    throw e;//just let it go, it is something else
  }
}

int server::queue_size(int socket) const {
  try {
    return indexer[socket].socket -> queue_size();//ask for the size of queue
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);
    }
    throw e;
  }
}

void server::set_onqueueempty(queueemptyevent e) {
  queue_empty_event = e;
}

queueemptyevent server::get_onqueueempty() const {
  return queue_empty_event;
}

void server::set_ondatasent(datasentevent e) {
  data_sent_event = e;
}

datasentevent server::get_ondatasent() const {
  return data_sent_event;
}

void server::write_data(int socket, int size, const void *data) {
  try {
    indexer[socket].socket -> write(size, (const char *) data);//write it to the socket and care no more
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);
    }
    throw e;
  }
}

void server::set_udp_local(int socket, int port) {
  try {
    int index = indexer[socket].index;//get the socket's index
    test_for_udp(index);//test it, weather it is udp and elsewhen throw an exception
    ((udp_socket *) sockets[index]) -> set_local(port);//set its local port
  }
  catch (tree_e *e) {
    if (e -> id == 1) {//the wanted socket does not exist
      delete e;
      throw new socket_e(22, "this is not my socket", socket);
    }
    throw e;//it is something else, let it go
  }
}

void server::set_udp_destination(int socket, const address &addr) {
  try {
    int index = indexer[socket].index;
    test_for_udp(index);
    ((udp_socket *) sockets[index]) -> set_remote(addr);//set the remote address
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "This is not my socket", socket);
    }
    throw e;
  }
}

address server::get_udp_source(int socket) {
  try {
    int index = indexer[socket].index;
    test_for_udp(index);
    return ((udp_socket *) sockets[index]) -> get_source_addr();//get the source address
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "This is not my socket", socket);
    }
    throw e;
  }
}

void server::write_data(int socket, const char *format, ...) {
  try {
    char *buff = (char *) malloc(10241);//10 K + 1 (NUL)
    va_list args;
    va_start(args, format);//initialize that args
    vsprintf(buff, format, args);//make that string
    va_end(args);//they are no longer needed
    indexer[socket].socket -> write(strlen(buff), buff);//write that data
    free(buff);//free that buffer
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(22, "not a socket", socket);
    }
    throw e;
  }
}

void server::close_socket(int socket) {
  delsock(socket, dsr_local_closed);//just remove the socket
  if (!running) delete_all();//it's being deleted at a stoped server, we can stop it now
}

int server::add_udp_socket(address_family af) {//add new udp socket
  return addsock(new udp_socket(af));//create new socket, add it there and return its id
}

void server::delete_all() {
  try {
    for (int i = 0; i < socketcount; i++) {//run trough them
      if (sockets[i] -> to_delete) {//delete this one
        indexer.remove(sockets[i] -> gethandle());//remove the socket from indexer
        delete sockets[i];//remove it at last
        if (i < socketcount - 1) {//it is not the last
          int last_sock = sockets[socketcount - 1] -> gethandle();//get handle of the last one
          indexer[last_sock].index = i;//move it here
          sockets[i] = sockets[socketcount - 1];//move the last here, I really don't care about sequence of them
        }
        if (prealloc - 10 < --socketcount) {
          sockets = (server_socket **) realloc(sockets, (prealloc = socketcount) * sizeof *sockets);//shrink it to the socketcount          
        }
        i--;//this is not too clean, however, I need to check this cell again, there is something else inside
      }
    }
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new socket_e(-1, "socket not in tree");
    }
    throw e;
  }
}

void server::test_for_udp(int index) {//if this does not throw an exception, it is an udp socket
  if (sockets[index] -> gettype() != sst_udp) throw new socket_e(27, "not a udp socket", sockets[index] -> gethandle());
}

address::address() {//default constructor of address, do not use directly
  //just set nothing
}

address::address(const std::string &_addr, int port) {
  struct hostent *ipval = gethostbyname(_addr.c_str());//get the name
  if (!ipval) {
    std::string errstr = "could not loogup host: ";
    errstr += strerror(H_ERRNO);//get the string
    throw new socket_e(4, errstr);
  }
  addr.af = ipval -> h_addrtype;//get the address type - is it AF_INET or AF_INET6
  if ((addr.af != AF_INET)
#ifndef VORLIB_NO_IPV6
   && (addr.af != AF_INET6)
#endif
   ) {
    throw new socket_e(28, "unknown address family");
  }
#ifndef VORLIB_NO_IPV6
  if (addr.af == AF_INET6) {
    ((ipv6_addr *) &addr) -> port = htons(port);//set the port
    ((ipv6_addr *) &addr) -> flow_info = 0;//set this to 0, I don't know whats that for
    memset( ((ipv6_addr *) &addr) -> address, 0, sizeof ((ipv6_addr *) &addr) -> address);//set it to 0, in the case that it will not be the full address and length will be shorter
    memcpy( ((ipv6_addr *) &addr) -> address, ipval -> h_addr_list[0], ipval -> h_length);//copy the address
    ((ipv6_addr *) &addr) -> scope_info = 0;//set this thing to 0 as well
    memset( ((ipv6_addr *) &addr) -> zero, 0, sizeof ((ipv6_addr *) &addr) -> zero);//zero the rest
  } else
#endif
  {
    ((ipv4_addr *) &addr) -> port = htons(port);//set the port
    memcpy( &((ipv4_addr *) &addr) -> address, ipval -> h_addr_list[0], 4);//copy the frst 4 bytes from there
    memset( ((ipv4_addr *) &addr) -> zero, 0, sizeof ((ipv4_addr *) &addr) -> zero);//zero the rest
  }
}

address::address(char _1, char _2, char _3, char _4, int port) {
  addr.af = AF_INET;//if it's 4 bytes long, it is just ipv4
  ((ipv4_addr *) &addr) -> port = htons(port);//set the port
  union {
    unsigned uaddr;//the address as it is
    struct {
      char _1, _2, _3, _4;//and the address byte by byte
    } x;
  } convertor;
  convertor.x._1 = _1;
  convertor.x._2 = _2;
  convertor.x._3 = _3;
  convertor.x._4 = _4;//put the address together
  ((ipv4_addr *) &addr) -> address = convertor.uaddr;//and the whole part put there
  memset( ((ipv4_addr *) &addr) -> zero, 0, sizeof ((ipv4_addr *) &addr) -> zero);//set the rest to 0
}

address::address(unsigned _addr, int port) {
  addr.af = AF_INET;//it can be converted only into ipv4
  ((ipv4_addr *) &addr) -> port = htons(port);//set the port
  ((ipv4_addr *) &addr) -> address = htonl(_addr);//set the address
  memset( ((ipv4_addr *) &addr) -> zero, 0, sizeof ((ipv4_addr *) &addr) -> zero);//set the rest to 0
}

address::address(const long_sockaddr *_addr, int length) {
  if (length < 2) throw new socket_e(23, "address data too short, not even the address family there");
  if ((_addr -> af != AF_INET)
#ifndef VORLIB_NO_IPV6
  && (_addr -> af != AF_INET6)
#endif
  ) throw new socket_e(28, "unknown address family");
  memset(&addr, 0, sizeof addr);//set it to 0, he may provide less data
  memcpy(&addr, _addr, sizeof addr);//copy what he gave us
}

address_family address::get_af() const {
#ifndef VORLIB_NO_IPV6
  if (addr.af == AF_INET6) return af_ipv6; else
#endif
  return af_ipv4;
}

const long_sockaddr *address::get_addr() const {
  return &addr;//return the address
}

unsigned address::get_un_addr() const {
  if (addr.af != AF_INET) throw new socket_e(28, "to call get_uaddr, you have to be shure that it is af_ipv4 address");
  return ntohl( ((ipv4_addr *) &addr) -> address);//novert the addres and give it him
}

int address::get_port() const {
  return ntohs(((ipv4_addr *) &addr) -> port);//they have it on the same place in the structure
}

std::string address::get_str_addr() const {
  char res[51];
  void *a;
#ifndef VORLIB_NO_IPV6
  if (addr.af == AF_INET6) {
    a = &((ipv6_addr *) &addr) -> address;//get the pointer to the address
  } else 
#endif
  a = &((ipv4_addr *) &addr) -> address;//get the pointer
#ifdef VORLIB_UNIX
  std::string result = inet_ntop(addr.af, a, res, 50);//convert the address
#endif
#ifdef VORLIB_WINDOWS
  std::string result = inet_ntoa(* ( (in_addr *) &addr.data));//that looks awfull and it is
  //it WONT work with ipv6 addreses
#endif
  return result;//and return it
}

address getaddr(const std::string &addr, int port) {
  return address(addr, port);//create it and give it to him
}

address getaddr(unsigned addr, int port) {
  return address(addr, port);
}

address getaddr(char _1, char _2, char _3, char _4, int port) {
  return address(_1, _2, _3, _4, port);
}

address getaddr(const long_sockaddr *addr, int length) {
  return address(addr, length);
}

}
