/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_STACK_H
#define VORLIB_STACK_H

#include "exception.h"

namespace vorlib {

template <class t> class stack {
private:
  int alloc_by;
  int allocated;
  int size;
  t **data;
public:
  stack(int alloc_by = 50);//default constructor
  stack(const stack &);//copying constructor
  ~stack();//destructor
  stack &operator=(const stack &);

  void push(const t &);
  stack &operator<<(const t &);

  t pop();
  stack &operator>>(t &);

  void clear();

  bool empty();

  t &top();
  const t &top() const;
};

template <class t> stack<t>::stack(int _alloc_by) {
  alloc_by = _alloc_by;
  allocated = 0;
  size = 0;
  data = NULL;//we have nothing yet
}

template <class t> stack<t>::stack(const stack<t> &source) {
  alloc_by = source.alloc_by;
  allocated = source.allocated;
  size = source.allocated;//copy properties
  data = (t **) malloc(allocated * sizeof *data);//make the array
  t **end = data + size;//how much do we copy?
  const t **src = source.data;//it's the souurce of the other data
  for (t **i = data; i != end; i++, src++) //copy them all
   *i = new t(**src);//copy the item
}

template <class t> stack<t>::~stack() {
  clear();
  free(data);
}

template <class t> stack<t> &stack<t>::operator=(const stack<t> &source) {
  t **end = data + size;
  for (t** i = data; i != data; i++) delete *i;//free all old items, if there are some
  size = source.size;
  allocated = source.allocated;
  alloc_by = source.alloc_by;//copy properties
  data = (t **) realloc(data, allocated * sizeof *data);//resize the array
  const t **src = source.data;//it's the souurce of the other data
  for (t **i = data; i != end; i++, src++) //copy them all
   *i = new t(**src);//copy the item
  return *this;
}

template <class t> void stack<t>::push(const t &item) {
  if (size == allocated) {//we are full, so make us bigger
    allocated += alloc_by;
    data = (t **) realloc(data, allocated * sizeof *data);
  }
  data[size++] = new t(item);//copy the item
}

template <class t> stack<t> &stack<t>::operator<<(const t &item) {
  push(item);
  return *this;
}

template <class t> t stack<t>::pop() {
  t res = *data[--size];//get the item somewhere
  delete data[size];//delete it in the array
  if (size - alloc_by == allocated) {
    size -= allocated;
    data = (t **) realloc(data, allocated * sizeof *data);//it can be shrinked now
  }
  return res;//and return the wanted item
}

template <class t> stack<t> &stack<t>::operator>>(t &result) {
  result = pop();
  return *this;
}

template <class t> void stack<t>::clear() {
  t **end = data + size;
  for (t **i = data; i != end; i++) delete *i;//remove them all
  free(data);
  data = NULL;
  size = 0;
  allocated = 0;//make it empty
}

template <class t> bool stack<t>::empty() {
  return (size == 0);
}

template <class t> t &stack<t>::top() {
  return *data[size - 1];
}

template <class t> const t &stack<t>::top() const {
  return *data[size - 1];
}

}

#endif
