/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_STRINGLIST_H
#define VORLIB_STRINGLIST_H

#include <string>
#include <vector>
using std::string;//the stringlist obviously uses string :)

namespace vorlib {

class stringlist {
private:
  int string_count;
  string **strings;
public:
  stringlist();//default constructor
  stringlist(const stringlist&);//copying constructor
  stringlist(const std::vector<std::string> &);//convert constructor
  ~stringlist();//destructor

  stringlist &operator=(stringlist &source);//assignment

  int size() const;//how many of them are inside?
  string &operator[](int index);
  const string &operator[](int index) const;//give me one of them

  void add(const string &value);
  stringlist &operator+=(const string &value);//adds some new string
  void join(const stringlist &source);
  stringlist &operator+=(const stringlist &source);//add all strings from some other stringlist
  stringlist &operator+(const string &value) const;
  stringlist &operator+(const stringlist &source) const;//addition with string, stringlist
  void insert(int index, const string &value);//put it somewhere not to the end

  int find(const string &value) const;
  int operator%(const string &value) const;//returns index of firs such string, if not there --> -1
  int count() const;

  bool remove(const string &value);
  bool remove(int index);//removes the wanted item (if is there) and returns weather it was there
  stringlist &operator-(const string &value) const;
  stringlist &operator-(int index) const;
  stringlist &operator-=(const string &value);
  stringlist &operator-=(int index);//like above
  bool remove_last();//removes the last item
  void clear();//removes all
};

}

#endif
