/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "textloader.h"
#include <string.h>
#include <stdio.h>

using std::string;

namespace vorlib {

textloader_e::textloader_e(int id, const string &message) : exception (id, message){
}

void textloader::clear() {
  files.clear();//clear all :))
}

const string &textloader::get_text(const string &file, const string &text) const {
  try {
    return files[file][text];//just get it
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new textloader_e(7, "file or text does not exist");//just replace the exception
    } else throw e;
  }
}

string &textloader::get_text(const string &file, const string &text) {
  try {
    return files[file][text];
  }
  catch (tree_e *e) {
    if (e -> id == 1) {
      delete e;
      throw new textloader_e(7, "file or text does not exist");
    } else throw e;
  }
}

textloader &textloader::operator<<(const string &file) {
  load(file);
  return *this;
}

void textloader::load(const string &file) {
  FILE *f = fopen(file.c_str(), "r");
  if (!f) throw new textloader_e(1, "file not found: " + file);//get the file
  char line[102400];//10 kB should be enough
  char *l;
  string text = "";//no text
  string name = "";//no name
  string filename = "";//we don't know the name of file yet
  file_t actual;//create new file storage
  while ((l = fgets(line, 12400, f)) != NULL) {//read line-by-line
    switch (*l) {
      case 0:
      case '\n'://the line is empty
      case '#'://it's a comment
        continue;//skiping this line
      case '~'://it's continuing text
        if (l[strlen(l) - 1] == '\n') l[strlen(l) - 1] = 0;//remove the newline at the end
        text += l;//add it to the text
        break;
      default:
        if (l[strlen(l) - 1] == '\n') l[strlen(l) - 1] = 0;//remove the newline at the end
        if (filename == "") {
          filename = l;//it's the first line >> its name of this file
          break;//I won't take it as a text as well
        }
        for (char *c = l; *c; c++) if (*c == ' ') {
          if (text != "")
            if (!actual.add(name, text)) throw new textloader_e(4, "duplicit text name in file");//add it there
          *c = 0;
          name = l;//copy name there
          text = (c + 1);//and the rest is the text
          goto FOUND_TEXT;//we are done
        }
        throw new textloader_e(3, "text not found after name");
        FOUND_TEXT:
        break;
    }
  }
  if (text != "")
    if (!actual.add(name, text)) throw new textloader_e(4, "duplicit text name in file");
  if (filename == "") throw new textloader_e(5, "no file name found");
  else if (!files.add(filename, actual)) throw new textloader_e(6, "file name duplicit");//add the whole file
  if (fclose(f)) throw new textloader_e(2, "file not closed: " + file);//the file is not needed now
}

}
